@@ -150,6 +150,38 @@ sync_folders_data_free (gpointer data)
 	g_free (sfd);
 }
 
+static void
+mapi_backend_update_enabled (ESource *data_source,
+			     ESource *collection_source)
+{
+	ESourceCollection *collection_extension = NULL;
+	gboolean part_enabled = TRUE;
+
+	g_return_if_fail (E_IS_SOURCE (data_source));
+
+	if (!collection_source || !e_source_get_enabled (collection_source)) {
+		e_source_set_enabled (data_source, FALSE);
+		return;
+	}
+
+	if (e_source_has_extension (collection_source, E_SOURCE_EXTENSION_COLLECTION))
+		collection_extension = e_source_get_extension (collection_source, E_SOURCE_EXTENSION_COLLECTION);
+
+	if (e_source_has_extension (data_source, E_SOURCE_EXTENSION_CALENDAR) ||
+	    e_source_has_extension (data_source, E_SOURCE_EXTENSION_TASK_LIST) ||
+	    e_source_has_extension (data_source, E_SOURCE_EXTENSION_MEMO_LIST)) {
+		part_enabled = !collection_extension || e_source_collection_get_calendar_enabled (collection_extension);
+	} else if (e_source_has_extension (data_source, E_SOURCE_EXTENSION_ADDRESS_BOOK)) {
+		part_enabled = !collection_extension || e_source_collection_get_contacts_enabled (collection_extension);
+	} else if (e_source_has_extension (data_source, E_SOURCE_EXTENSION_MAIL_ACCOUNT) ||
+		   e_source_has_extension (data_source, E_SOURCE_EXTENSION_MAIL_IDENTITY) ||
+		   e_source_has_extension (data_source, E_SOURCE_EXTENSION_MAIL_TRANSPORT)) {
+		part_enabled = !collection_extension || e_source_collection_get_mail_enabled (collection_extension);
+	}
+
+	e_source_set_enabled (data_source, part_enabled);
+}
+
 static gboolean
 mapi_backend_sync_folders_idle_cb (gpointer user_data)
 {
@@ -197,7 +229,7 @@ mapi_backend_sync_folders_idle_cb (gpointer user_data)
 
 		source = e_mapi_utils_get_source_for_folder (configured, sfd->profile, e_mapi_folder_get_id (folder));
 		if (source) {
-			e_source_set_enabled (source, TRUE);
+			mapi_backend_update_enabled (source, e_backend_get_source (E_BACKEND (backend)));
 
 			if (g_strcmp0 (e_source_get_display_name (source), e_mapi_folder_get_name (folder)) != 0)
 				e_source_set_display_name (source, e_mapi_folder_get_name (folder));
@@ -231,7 +263,7 @@ mapi_backend_sync_folders_idle_cb (gpointer user_data)
 				NULL,
 				NULL)) {
 				color_seed++;
-				e_source_set_enabled (source, TRUE);
+				mapi_backend_update_enabled (source, e_backend_get_source (E_BACKEND (backend)));
 				e_server_side_source_set_writable (E_SERVER_SIDE_SOURCE (source), TRUE);
 				e_server_side_source_set_remote_deletable (E_SERVER_SIDE_SOURCE (source), TRUE);
 				e_source_registry_server_add_source (server, source);
@@ -831,7 +863,7 @@ mapi_backend_authenticate_sync (EBackend *backend,
 	EMapiConnection *conn;
 	CamelMapiSettings *settings;
 	GSList *mapi_folders = NULL;
-	GError *mapi_error = NULL;
+	GError *mapi_error = NULL, *krb_error = NULL;
 
 	g_return_val_if_fail (E_IS_MAPI_BACKEND (backend), E_SOURCE_AUTHENTICATION_ERROR);
 
@@ -839,7 +871,7 @@ mapi_backend_authenticate_sync (EBackend *backend,
 	settings = mapi_backend_get_settings (mapi_backend);
 
 	if (camel_mapi_settings_get_kerberos (settings))
-		e_mapi_util_trigger_krb_auth_from_settings (settings, NULL);
+		e_mapi_util_trigger_krb_auth_from_settings (settings, &krb_error);
 
 	conn = e_mapi_connection_new (NULL,
 		camel_mapi_settings_get_profile (settings),
@@ -854,10 +886,22 @@ mapi_backend_authenticate_sync (EBackend *backend,
 		    g_error_matches (mapi_error, E_MAPI_ERROR, MAPI_E_PASSWORD_EXPIRED))
 			res = E_SOURCE_AUTHENTICATION_REJECTED;
 
-		if (res != E_SOURCE_AUTHENTICATION_REJECTED)
+		if (res != E_SOURCE_AUTHENTICATION_REJECTED) {
+			if (krb_error) {
+				GError *new_error = g_error_new (mapi_error->domain, mapi_error->code,
+					/* Translators: the first '%s' is replaced with a generic error message,
+					   the second '%s' is replaced with additional error information. */
+					C_("gssapi_error", "%s (%s)"), mapi_error->message, krb_error->message);
+				g_clear_error (&mapi_error);
+				mapi_error = new_error;
+			}
+
 			g_propagate_error (error, mapi_error);
-		else
+		} else {
 			g_clear_error (&mapi_error);
+		}
+
+		g_clear_error (&krb_error);
 
 		return res;
 	}
@@ -892,6 +936,7 @@ mapi_backend_authenticate_sync (EBackend *backend,
 
 	g_object_unref (conn);
 	g_clear_error (&mapi_error);
+	g_clear_error (&krb_error);
 
 	return E_SOURCE_AUTHENTICATION_ACCEPTED;
 }
